package java.util.jar;

import sun.misc.ASCIICaseInsensitiveComparator;
import sun.util.logging.PlatformLogger;
import java.io.IOException;
import java.io.DataOutputStream;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

public class Attributes implements Map<Object, Object>, Cloneable
{
    protected Map<Object, Object> map;
    
    public Attributes() {
        this(11);
    }
    
    public Attributes(int initialCapacity) {
        this.map = new HashMap(initialCapacity);
    }
    
    public Attributes(Attributes m) {
        this.map = new HashMap(m);
    }
    
    @Override
    public Object get(Object o) {
        return this.map.get(o);
    }
    
    public String getValue(String s) {
        return (String)this.get(new Name(s));
    }
    
    public String getValue(Attributes.Name name) {
        return (String)this.get(name);
    }
    
    @Override
    public Object put(Object o, Object o2) {
        return this.map.put((Object)o, (Object)o2);
    }
    
    public String putValue(String s, String s2) {
        return (String)this.put(new Name(s), s2);
    }
    
    @Override
    public Object remove(Object o) {
        return this.map.remove(o);
    }
    
    @Override
    public boolean containsValue(Object o) {
        return this.map.containsValue(o);
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.map.containsKey(o);
    }
    
    @Override
    public void putAll(Map<?, ?> map) {
        if (!Attributes.class.isInstance(map)) {
            throw new ClassCastException();
        }
        for (Map.Entry entry : map.entrySet()) {
            this.put(entry.getKey(), entry.getValue());
        }
        return;
    }
    
    @Override
    public void clear() {
        this.map.clear();
    }
    
    @Override
    public int size() {
        return this.map.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.map.isEmpty();
    }
    
    @Override
    public Set<Object> keySet() {
        return this.map.keySet();
    }
    
    @Override
    public Collection<Object> values() {
        return this.map.values();
    }
    
    @Override
    public Set<Map.Entry<Object, Object>> entrySet() {
        return this.map.entrySet();
    }
    
    @Override
    public boolean equals(Object o) {
        return this.map.equals(o);
    }
    
    @Override
    public int hashCode() {
        return this.map.hashCode();
    }
    
    public Object clone() {
        return new Attributes(this);
    }
    
    void write(DataOutputStream dataOutputStream) throws IOException {
        for (Map.Entry entry : this.entrySet()) {
            StringBuffer o = new StringBuffer(((Attributes.Name)entry.getKey()).toString());
            o.append(": ");
            String str = (Object)entry.getValue();
            if (str != null) {
                byte[] bytes = str.getBytes("UTF8");
                str = new String(bytes, 0, 0, bytes.length);
            }
            o.append(str);
            o.append("\r\n");
            Manifest.make72Safe(o);
            dataOutputStream.writeBytes(o.toString());
        }
        dataOutputStream.writeBytes("\r\n");
    }
    
    void writeMain(DataOutputStream dataOutputStream) throws IOException {
        String s = Attributes.Name.MANIFEST_VERSION.toString();
        String str = this.getValue(s);
        if (str == null) {
            s = Attributes.Name.SIGNATURE_VERSION.toString();
            str = this.getValue(s);
        }
        if (str != null) {
            dataOutputStream.writeBytes(s + ": " + str + "\r\n");
        }
        for (Map.Entry entry : this.entrySet()) {
            String string = ((Attributes.Name)entry.getKey()).toString();
            if (str != null && !string.equalsIgnoreCase(s)) {
                StringBuffer o = new StringBuffer(string);
                o.append(": ");
                String str2 = (Object)entry.getValue();
                if (str2 != null) {
                    byte[] bytes = str2.getBytes("UTF8");
                    str2 = new String(bytes, 0, 0, bytes.length);
                }
                o.append(str2);
                o.append("\r\n");
                Manifest.make72Safe(o);
                dataOutputStream.writeBytes(o.toString());
            }
            else {
                continue;
            }
        }
        dataOutputStream.writeBytes("\r\n");
    }
    
    void read(Manifest.FastInputStream fastInputStream, byte[] array) throws IOException {
        Object o = null;
        byte[] array2 = null;
        int line;
        while ((line = fastInputStream.readLine(array)) != -1) {
            boolean n = false;
            if (array[--line] != 10) {
                throw new IOException("line too long");
            }
            if (line > 0 && array[line - 1] == 13) {
                --line;
            }
            if (line == 0) {
                break;
            }
            int offset = 0;
            Object o2;
            if (array[0] == 32) {
                if (o == null) {
                    throw new IOException("misplaced continuation line");
                }
                n = true;
                byte[] bytes = new byte[array2.length + line - 1];
                System.arraycopy(array2, 0, bytes, 0, array2.length);
                System.arraycopy(array, 1, bytes, array2.length, line - 1);
                if (fastInputStream.peek() == 32) {
                    array2 = bytes;
                    continue;
                }
                o2 = new String(bytes, 0, bytes.length, "UTF8");
                array2 = null;
            }
            else {
                while (array[offset++] != 58) {
                    if (offset < line) {
                        continue;
                    }
                    throw new IOException("invalid header field");
                }
                if (array[offset++] != 32) {
                    throw new IOException("invalid header field");
                }
                o = new String(array, 0, 0, offset - 2);
                if (fastInputStream.peek() == 32) {
                    array2 = new byte[line - offset];
                    System.arraycopy(array, offset, array2, 0, line - offset);
                    continue;
                }
                o2 = new String(array, offset, line - offset, "UTF8");
            }
            try {
                if (this.putValue(o, o2) != null && n == 0) {
                    PlatformLogger.getLogger("java.util.jar").warning("Duplicate name in Manifest: " + o + ".\nEnsure that the manifest does not have duplicate entries, and\nthat blank lines separate individual sections in both your\nmanifest and in the META-INF/MANIFEST.MF entry in the jar file.");
                }
                else {
                    continue;
                }
            }
            catch (IllegalArgumentException ex) {
                throw new IOException("invalid header field name: " + o);
            }
        }
    }
    
    public static class Name
    {
        private String name;
        private int hashCode;
        public static final Attributes.Name MANIFEST_VERSION;
        public static final Attributes.Name SIGNATURE_VERSION;
        public static final Attributes.Name CONTENT_TYPE;
        public static final Attributes.Name CLASS_PATH;
        public static final Attributes.Name MAIN_CLASS;
        public static final Attributes.Name SEALED;
        public static final Attributes.Name EXTENSION_LIST;
        public static final Attributes.Name EXTENSION_NAME;
        @Deprecated
        public static final Attributes.Name EXTENSION_INSTALLATION;
        public static final Attributes.Name IMPLEMENTATION_TITLE;
        public static final Attributes.Name IMPLEMENTATION_VERSION;
        public static final Attributes.Name IMPLEMENTATION_VENDOR;
        @Deprecated
        public static final Attributes.Name IMPLEMENTATION_VENDOR_ID;
        @Deprecated
        public static final Attributes.Name IMPLEMENTATION_URL;
        public static final Attributes.Name SPECIFICATION_TITLE;
        public static final Attributes.Name SPECIFICATION_VERSION;
        public static final Attributes.Name SPECIFICATION_VENDOR;
        
        public Name(String s) {
            this.hashCode = -1;
            if (s == null) {
                throw new NullPointerException("name");
            }
            if (!isValid(s)) {
                throw new IllegalArgumentException(s);
            }
            this.name = s.intern();
            return;
        }
        
        private static boolean isValid(String s) {
            int length = s.length();
            if (length > 70 || length == 0) {
                return false;
            }
            int i = 0;
            while (i < length) {
                if (!isValid(s.charAt(i))) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        
        private static boolean isValid(char c) {
            return isAlpha(c) || isDigit(c) || c == '_' || c == '-';
        }
        
        private static boolean isAlpha(char c) {
            return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z');
        }
        
        private static boolean isDigit(char c) {
            return c >= '0' && c <= '9';
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof Attributes.Name && ASCIICaseInsensitiveComparator.CASE_INSENSITIVE_ORDER.compare(this.name, ((Attributes.Name)o).name) == 0;
        }
        
        @Override
        public int hashCode() {
            if (this.hashCode == -1) {
                this.hashCode = ASCIICaseInsensitiveComparator.lowerCaseHashCode(this.name);
            }
            return this.hashCode;
        }
        
        @Override
        public String toString() {
            return this.name;
        }
        
        static {
            MANIFEST_VERSION = new Attributes.Name("Manifest-Version");
            SIGNATURE_VERSION = new Attributes.Name("Signature-Version");
            CONTENT_TYPE = new Attributes.Name("Content-Type");
            CLASS_PATH = new Attributes.Name("Class-Path");
            MAIN_CLASS = new Attributes.Name("Main-Class");
            SEALED = new Attributes.Name("Sealed");
            EXTENSION_LIST = new Attributes.Name("Extension-List");
            EXTENSION_NAME = new Attributes.Name("Extension-Name");
            EXTENSION_INSTALLATION = new Attributes.Name("Extension-Installation");
            IMPLEMENTATION_TITLE = new Attributes.Name("Implementation-Title");
            IMPLEMENTATION_VERSION = new Attributes.Name("Implementation-Version");
            IMPLEMENTATION_VENDOR = new Attributes.Name("Implementation-Vendor");
            IMPLEMENTATION_VENDOR_ID = new Attributes.Name("Implementation-Vendor-Id");
            IMPLEMENTATION_URL = new Attributes.Name("Implementation-URL");
            SPECIFICATION_TITLE = new Attributes.Name("Specification-Title");
            SPECIFICATION_VERSION = new Attributes.Name("Specification-Version");
            SPECIFICATION_VENDOR = new Attributes.Name("Specification-Vendor");
        }
    }
}
